"""
K6GTE, Database class to store contacts
Email: michael.bridak@gmail.com
GPL V3
"""
import logging
import sqlite3

if __name__ == "__main__":
    print("I'm not the program you are looking for.")


class DataBase:
    """Database class for our database."""

    def __init__(self, database: str):
        """initializes DataBase instance"""
        self.logger = logging.getLogger("__name__")
        self.empty_contact = {
            "primarykey": 1,
            "app": "",
            "contestname": "",
            "contestnr": "",
            "timestamp": "",
            "mycall": "",
            "band": "",
            "rxfreq": "",
            "txfreq": "",
            "operator": "",
            "mode": "",
            "call": "",
            "countryprefix": "",
            "wpxprefix": "",
            "stationprefix": "",
            "continent": "",
            "snt": "",
            "sntnr": "",
            "rcv": "",
            "rcvnr": "",
            "gridsquare": "",
            "exchangel": "",
            "section": "",
            "comment": "",
            "qth": "",
            "name": "",
            "power": "",
            "misctext": "",
            "zone": "",
            "prec": "",
            "ck": "",
            "ismultiplierl": "",
            "ismultiplier2": "",
            "ismultiplier3": "",
            "points": "",
            "radionr": "",
            "run1run2": "",
            "RoverLocation": "",
            "RadioInterfaced": "",
            "NetworkedCompNr": "",
            "IsOriginal": "",
            "NetBiosName": "",
            "IsRunQSO": "",
            "StationName": "",
            "ID": "",
            "IsClaimedQso": 1,
        }
        self.database = database
        self.create_db()

    @staticmethod
    def row_factory(cursor, row):
        """
        cursor.description:
        (name, type_code, display_size,
        internal_size, precision, scale, null_ok)
        row: (value, value, ...)
        """
        return {
            col[0]: row[idx]
            for idx, col in enumerate(
                cursor.description,
            )
        }

    def create_db(self) -> None:
        """create a database and table if it does not exist"""
        self.logger.info("Creating Database: %s", self.database)
        with sqlite3.connect(self.database) as conn:
            cursor = conn.cursor()
            sql_table = (
                "CREATE TABLE IF NOT EXISTS contactinfo "
                "(primarykey INTEGER PRIMARY KEY, "
                "app TEXT DEFAULT K6GTELOGGER,"
                "contestname TEXT,"
                "contestnr TEXT,"
                "timestamp TEXT,"
                "mycall TEXT,"
                "band TEXT,"
                "rxfreq TEXT,"
                "txfreq TEXT,"
                "operator TEXT,"
                "mode TEXT,"
                "call TEXT,"
                "countryprefix TEXT,"
                "wpxprefix TEXT,"
                "stationprefix TEXT,"
                "continent TEXT,"
                "snt TEXT,"
                "sntnr TEXT,"
                "rcv TEXT,"
                "rcvnr TEXT,"
                "gridsquare TEXT,"
                "exchangel TEXT,"
                "section TEXT,"
                "comment TEXT,"
                "qth TEXT,"
                "name TEXT,"
                "power TEXT,"
                "misctext TEXT,"
                "zone TEXT,"
                "prec TEXT,"
                "ck TEXT,"
                "ismultiplierl TEXT,"
                "ismultiplier2 TEXT,"
                "ismultiplier3 TEXT,"
                "points TEXT,"
                "radionr TEXT,"
                "run1run2 TEXT,"
                "RoverLocation TEXT,"
                "RadioInterfaced TEXT,"
                "NetworkedCompNr TEXT,"
                "IsOriginal TEXT,"
                "NetBiosName TEXT,"
                "IsRunQSO TEXT,"
                "StationName TEXT,"
                "ID TEXT UNIQUE,"
                "IsClaimedQso TEXT,"
                "dirty INTEGER DEFAULT 1);"
            )
            cursor.execute(sql_table)
            conn.commit()

    def log_contact(self, contact: dict) -> None:
        """
        Inserts a contact into the db.
        pass in a dict object, see get_empty() for keys
        """
        self.logger.info("%s", contact)

        pre = "INSERT INTO contactinfo("
        values = []
        columns = ""
        placeholders = ""
        for key in contact.keys():
            columns += f"{key},"
            values.append(contact[key])
            placeholders += "?,"
        post = f") VALUES({placeholders[:-1]});"
        sql = f"{pre}{columns[:-1]}{post}"

        try:
            with sqlite3.connect(self.database) as conn:
                self.logger.info("%s", sql)
                cur = conn.cursor()
                cur.execute(sql, tuple(values))
                conn.commit()
        except sqlite3.Error as exception:
            self.logger.info("DataBase log_contact: %s", exception)

    def change_contact(self, qso: dict) -> None:
        """Update an existing contact."""

        pre = "UPDATE contactinfo set "
        for key in qso.keys():
            pre += f"{key} = '{qso[key]}',"
        sql = f"{pre[:-1]} where primarykey='{qso['primarykey']}';"

        try:
            with sqlite3.connect(self.database) as conn:
                self.logger.info("%s\n%s", sql, qso)
                cur = conn.cursor()
                cur.execute(sql)
                conn.commit()
        except sqlite3.Error as exception:
            self.logger.info("DataBase change_contact: %s", exception)

    def get_unique_id(self, contact) -> str:
        """get unique id"""
        unique_id = ""
        if contact:
            try:
                with sqlite3.connect(self.database) as conn:
                    sql = f"select ID from contactinfo where primarykey={int(contact)}"
                    cursor = conn.cursor()
                    cursor.execute(sql)
                    unique_id = str(cursor.fetchone()[0])
            except sqlite3.Error as exception:
                self.logger.debug("%s", exception)
        return unique_id

    def delete_contact(self, contact) -> None:
        """Deletes a contact from the db."""
        if contact:
            try:
                with sqlite3.connect(self.database) as conn:
                    sql = f"delete from contactinfo where primarykey={int(contact)};"
                    cur = conn.cursor()
                    cur.execute(sql)
                    conn.commit()
            except sqlite3.Error as exception:
                self.logger.info("DataBase delete_contact: %s", exception)

    def fetch_all_contacts_asc(self) -> list:
        """returns a list of dicts with contacts in the database."""
        with sqlite3.connect(self.database) as conn:
            conn.row_factory = self.row_factory
            cursor = conn.cursor()
            cursor.execute("select * from contactinfo order by timestamp ASC;")
            return cursor.fetchall()

    def fetch_all_contacts_desc(self) -> list:
        """returns a list of dicts with contacts in the database."""
        with sqlite3.connect(self.database) as conn:
            conn.row_factory = self.row_factory
            cursor = conn.cursor()
            cursor.execute("select * from contactinfo order by timestamp desc;")
            return cursor.fetchall()

    def fetch_last_contact(self) -> dict:
        """returns a list of dicts with last contact in the database."""
        with sqlite3.connect(self.database) as conn:
            conn.row_factory = self.row_factory
            cursor = conn.cursor()
            cursor.execute("select * from contactinfo order by timestamp desc;")
            return cursor.fetchone()

    def fetch_all_dirty_contacts(self) -> list:
        """
        Return a list of dict, containing all contacts still flagged as dirty.\n
        """
        with sqlite3.connect(self.database) as conn:
            conn.row_factory = self.row_factory
            cursor = conn.cursor()
            cursor.execute("select * from contactinfo where dirty=1 order by id")
            return cursor.fetchall()

    def get_empty(self) -> dict:
        """Return a dictionary object with keys and no values."""
        return self.empty_contact



#!/usr/bin/env python3
"""
NOT1MM Logger
"""
# pylint: disable=unused-import, c-extension-no-member

import importlib
import logging
import os
import pkgutil
import re
import sqlite3
import sys
from datetime import datetime
from json import dumps, loads
from pathlib import Path
from shutil import copyfile
from xmlrpc.client import Error, ServerProxy

import psutil
from PyQt5 import QtCore, QtGui, QtWidgets, uic
from PyQt5.QtCore import QDir, Qt  # pylint: disable=no-name-in-module
from PyQt5.QtGui import QFontDatabase  # pylint: disable=no-name-in-module

try:
    from not1mm.lib.cat_interface import CAT
    from not1mm.lib.cwinterface import CW
    from not1mm.lib.ham_utility import *
    from not1mm.lib.lookup import QRZlookup
    from not1mm.lib.version import __version__
except ModuleNotFoundError:
    from lib.cat_interface import CAT
    from lib.cwinterface import CW
    from lib.ham_utility import *
    from lib.lookup import QRZlookup
    from lib.version import __version__


loader = pkgutil.get_loader("not1mm")
WORKING_PATH = os.path.dirname(loader.get_filename())


if "XDG_DATA_HOME" in os.environ:
    DATA_PATH = os.environ.get("XDG_DATA_HOME")
else:
    DATA_PATH = str(Path.home() / ".local" / "share")
DATA_PATH += "/not1mm"
try:
    os.mkdir(DATA_PATH)
except FileExistsError:
    ...

if "XDG_CONFIG_HOME" in os.environ:
    CONFIG_PATH = os.environ.get("XDG_CONFIG_HOME")
else:
    CONFIG_PATH = str(Path.home() / ".config")
CONFIG_PATH += "/not1mm"
try:
    os.mkdir(CONFIG_PATH)
except FileExistsError:
    ...

"/data/cty.json"

CTYFILE = {}

with open(WORKING_PATH + "/data/cty.json", "rt", encoding="utf-8") as fd:
    CTYFILE = loads(fd.read())


def cty_lookup(callsign: str):
    callsign = callsign.upper()
    for x in reversed(range(len(callsign))):
        searchitem = callsign[: x + 1]
        result = {key: val for key, val in CTYFILE.items() if key == searchitem}
        if not result:
            continue
        if result.get(searchitem).get("exact_match"):
            if searchitem == callsign:
                return result
            else:
                continue
        return result


class MainWindow(QtWidgets.QMainWindow):
    """
    The main window
    """

    pref_ref = {
        "useqrz": False,
        "lookupusername": "username",
        "lookuppassword": "password",
        "gridsquare": "AA11aa",
        "run_state": True,
        "dark_mode": False,
        "command_buttons": True,
        "cw_macros": True,
        "bands_modes": True,
    }
    contest = None
    pref = None
    current_op = ""
    current_mode = ""
    current_band = ""
    look_up = None
    run_state = False
    fkeys = {}

    def __init__(self, *args, **kwargs):
        logger.info("MainWindow: __init__")
        super().__init__(*args, **kwargs)
        data_path = WORKING_PATH + "/data/main.ui"
        uic.loadUi(data_path, self)
        self.readpreferences()
        self.next_field = self.other
        self.field4.hide()
        self.actionCW_Macros.triggered.connect(self.cw_macros_stateChanged)
        self.actionCommand_Buttons.triggered.connect(self.command_buttons_stateChange)
        self.actionMode_and_Bands.triggered.connect(self.show_band_mode_stateChange)
        self.actionDark_Mode.triggered.connect(self.dark_mode_stateChange)
        self.radioButton_run.clicked.connect(self.run_sp_buttons_clicked)
        self.radioButton_sp.clicked.connect(self.run_sp_buttons_clicked)
        self.score.setText("0")
        self.callsign.textEdited.connect(self.callsign_changed)
        self.sent.setText("59")
        self.receive.setText("59")
        icon_path = WORKING_PATH + "/data/"
        self.greendot = QtGui.QPixmap(icon_path + "greendot.png")
        self.reddot = QtGui.QPixmap(icon_path + "reddot.png")
        self.leftdot.setPixmap(self.greendot)
        self.rightdot.setPixmap(self.reddot)
        self.read_cw_macros()
        self.rig_control = CAT("rigctld", "localhost", 4532)
        self.F1.setContextMenuPolicy(QtCore.Qt.CustomContextMenu)
        self.F1.customContextMenuRequested.connect(self.edit_F1)
        self.F1.clicked.connect(self.sendf1)
        self.F2.setContextMenuPolicy(QtCore.Qt.CustomContextMenu)
        self.F2.customContextMenuRequested.connect(self.edit_F2)
        self.F2.clicked.connect(self.sendf2)
        self.F3.setContextMenuPolicy(QtCore.Qt.CustomContextMenu)
        self.F3.customContextMenuRequested.connect(self.edit_F3)
        self.F3.clicked.connect(self.sendf3)
        self.F4.setContextMenuPolicy(QtCore.Qt.CustomContextMenu)
        self.F4.customContextMenuRequested.connect(self.edit_F4)
        self.F4.clicked.connect(self.sendf4)
        self.F5.setContextMenuPolicy(QtCore.Qt.CustomContextMenu)
        self.F5.customContextMenuRequested.connect(self.edit_F5)
        self.F5.clicked.connect(self.sendf5)
        self.F6.setContextMenuPolicy(QtCore.Qt.CustomContextMenu)
        self.F6.customContextMenuRequested.connect(self.edit_F6)
        self.F6.clicked.connect(self.sendf6)
        self.F7.setContextMenuPolicy(QtCore.Qt.CustomContextMenu)
        self.F7.customContextMenuRequested.connect(self.edit_F7)
        self.F7.clicked.connect(self.sendf7)
        self.F8.setContextMenuPolicy(QtCore.Qt.CustomContextMenu)
        self.F8.customContextMenuRequested.connect(self.edit_F8)
        self.F8.clicked.connect(self.sendf8)
        self.F9.setContextMenuPolicy(QtCore.Qt.CustomContextMenu)
        self.F9.customContextMenuRequested.connect(self.edit_F9)
        self.F9.clicked.connect(self.sendf9)
        self.F10.setContextMenuPolicy(QtCore.Qt.CustomContextMenu)
        self.F10.customContextMenuRequested.connect(self.edit_F10)
        self.F10.clicked.connect(self.sendf10)
        self.F11.setContextMenuPolicy(QtCore.Qt.CustomContextMenu)
        self.F11.customContextMenuRequested.connect(self.edit_F11)
        self.F11.clicked.connect(self.sendf11)
        self.F12.setContextMenuPolicy(QtCore.Qt.CustomContextMenu)
        self.F12.customContextMenuRequested.connect(self.edit_F12)
        self.F12.clicked.connect(self.sendf12)
        self.select_contest()

    def select_contest(self):
        self.contest = doimp("arrl_field_day")
        logger.debug(f"Loaded Contest Name = {self.contest.name}")
        self.contest.interface(self)
        ...

    def edit_macro(self, function_key):
        self.edit_macro_dialog = EditMacro(function_key)
        self.edit_macro_dialog.accepted.connect(self.edited_macro)
        self.edit_macro_dialog.open()

    def edited_macro(self):
        self.edit_macro_dialog.function_key.setText(
            self.edit_macro_dialog.macro_label.text()
        )
        self.edit_macro_dialog.function_key.setToolTip(
            self.edit_macro_dialog.the_macro.text()
        )
        self.edit_macro_dialog.close()
        # logger.debug(f"{self.current_op}")

    def edit_F1(self):
        logger.debug("F1 Right Clicked.")
        self.edit_macro(self.F1)

    def edit_F2(self):
        logger.debug("F2 Right Clicked.")
        self.edit_macro(self.F2)

    def edit_F3(self):
        logger.debug("F3 Right Clicked.")
        self.edit_macro(self.F3)

    def edit_F4(self):
        logger.debug("F4 Right Clicked.")
        self.edit_macro(self.F4)

    def edit_F5(self):
        logger.debug("F5 Right Clicked.")
        self.edit_macro(self.F5)

    def edit_F6(self):
        logger.debug("F6 Right Clicked.")
        self.edit_macro(self.F6)

    def edit_F7(self):
        logger.debug("F7 Right Clicked.")
        self.edit_macro(self.F7)

    def edit_F8(self):
        logger.debug("F8 Right Clicked.")
        self.edit_macro(self.F8)

    def edit_F9(self):
        logger.debug("F9 Right Clicked.")
        self.edit_macro(self.F9)

    def edit_F10(self):
        logger.debug("F10 Right Clicked.")
        self.edit_macro(self.F10)

    def edit_F11(self):
        logger.debug("F11 Right Clicked.")
        self.edit_macro(self.F11)

    def edit_F12(self):
        logger.debug("F12 Right Clicked.")
        self.edit_macro(self.F12)

    def sendf1(self):
        logger.debug("F1 Clicked")

    def sendf2(self):
        logger.debug("F2 Clicked")

    def sendf3(self):
        logger.debug("F3 Clicked")

    def sendf4(self):
        logger.debug("F4 Clicked")

    def sendf5(self):
        logger.debug("F5 Clicked")

    def sendf6(self):
        logger.debug("F6 Clicked")

    def sendf7(self):
        logger.debug("F7 Clicked")

    def sendf8(self):
        logger.debug("F8 Clicked")

    def sendf9(self):
        logger.debug("F9 Clicked")

    def sendf10(self):
        logger.debug("F10 Clicked")

    def sendf11(self):
        logger.debug("F11 Clicked")

    def sendf12(self):
        logger.debug("F12 Clicked")

    def run_sp_buttons_clicked(self):
        self.pref["run_state"] = self.radioButton_run.isChecked()
        self.write_preference()
        self.read_cw_macros()

    def write_preference(self):
        """
        Write preferences to json file.
        """
        try:
            with open(
                CONFIG_PATH + "/not1mm.json", "wt", encoding="utf-8"
            ) as file_descriptor:
                file_descriptor.write(dumps(self.pref, indent=4))
                logger.info("writing: %s", self.pref)
        except IOError as exception:
            ...
            logger.critical("writepreferences: %s", exception)

    def readpreferences(self):
        """
        Restore preferences if they exist, otherwise create some sane defaults.
        """
        try:
            if os.path.exists(CONFIG_PATH + "/not1mm.json"):
                with open(
                    CONFIG_PATH + "/not1mm.json", "rt", encoding="utf-8"
                ) as file_descriptor:
                    self.pref = loads(file_descriptor.read())
                    logger.info("%s", self.pref)
            else:
                logger.info("No preference file. Writing preference.")
                with open(
                    CONFIG_PATH + "/not1mm.json", "wt", encoding="utf-8"
                ) as file_descriptor:
                    self.pref = self.pref_ref.copy()
                    file_descriptor.write(dumps(self.pref, indent=4))
                    logger.info("%s", self.pref)
        except IOError as exception:
            ...
            logger.critical("Error: %s", exception)
        if self.pref.get("useqrz"):
            self.look_up = QRZlookup(
                self.pref.get("lookupusername"),
                self.pref.get("lookuppassword"),
            )
        if self.pref.get("run_state"):
            self.radioButton_run.setChecked(True)
        else:
            self.radioButton_sp.setChecked(True)

        if self.pref.get("dark_mode"):
            self.actionDark_Mode.setChecked(True)
        else:
            self.actionDark_Mode.setChecked(False)

        if self.pref.get("command_buttons"):
            self.actionCommand_Buttons.setChecked(True)
        else:
            self.actionCommand_Buttons.setChecked(False)

        if self.pref.get("cw_macros"):
            self.actionCW_Macros.setChecked(True)
        else:
            self.actionCW_Macros.setChecked(False)

        if self.pref.get("bands_modes"):
            self.actionMode_and_Bands.setChecked(True)
        else:
            self.actionMode_and_Bands.setChecked(False)

        self.dark_mode()
        self.show_command_buttons()
        self.show_CW_macros()
        self.show_band_mode()

    def dark_mode_stateChange(self):
        self.pref["dark_mode"] = self.actionDark_Mode.isChecked()
        self.write_preference()
        self.dark_mode()

    def dark_mode(self):
        if self.pref.get("dark_mode"):
            with open(WORKING_PATH + "/data/Combinear.qss") as stylefile:
                qss = stylefile.read()
                self.setStyleSheet(qss)
        else:
            self.setStyleSheet("")

    def cw_macros_stateChanged(self):
        self.pref["cw_macros"] = self.actionCW_Macros.isChecked()
        self.write_preference()
        self.show_CW_macros()

    def show_CW_macros(self):
        if self.pref.get("cw_macros"):
            self.Button_Row1.show()
            self.Button_Row2.show()
        else:
            self.Button_Row1.hide()
            self.Button_Row2.hide()

    def command_buttons_stateChange(self):
        self.pref["command_buttons"] = self.actionCommand_Buttons.isChecked()
        self.write_preference()
        self.show_command_buttons()

    def show_command_buttons(self):
        if self.pref.get("command_buttons"):
            self.Command_Buttons.show()
        else:
            self.Command_Buttons.hide()

    def show_band_mode_stateChange(self):
        self.pref["bands_modes"] = self.actionMode_and_Bands.isChecked()
        self.write_preference()
        self.show_band_mode()

    def show_band_mode(self):
        if self.actionMode_and_Bands.isChecked():
            self.Band_Mode_Frame.show()
        else:
            self.Band_Mode_Frame.hide()

    def callsign_changed(self):
        text = self.callsign.text()
        text = text.upper()
        stripped_text = text.strip()
        self.callsign.setText(stripped_text)

        if text[-1:] == " ":
            if stripped_text == "CW":
                self.setmode("CW")
                self.callsign.setText("")
                return
            if stripped_text == "SSB":
                self.setmode("SSB")
                self.callsign.setText("")
                return
            if stripped_text == "OPON":
                self.get_opon()
                self.callsign.setText("")
                return
            self.check_callsign(text)
            self.check_callsign2(text)
            self.next_field.setFocus()

    def check_callsign(self, callsign):
        result = cty_lookup(callsign)

        print(result)
        for a in result.items():
            entity = a[1].get("entity")
            cq = a[1].get("cq")
            itu = a[1].get("itu")
            continent = a[1].get("continent")
            lat = float(a[1].get("lat"))
            lon = float(a[1].get("long"))
            lon = lon * -1  # cty.dat file inverts longitudes
            primary_pfx = a[1].get("primary_pfx")
            heading = bearing_with_latlon(self.pref.get("gridsquare"), lat, lon)
            kilometers = distance_with_latlon(self.pref.get("gridsquare"), lat, lon)
            self.heading_distance.setText(
                f"heading {heading}° / distance {int(kilometers*0.621371)}mi {kilometers}km"
            )
            self.dx_entity.setText(
                f"{primary_pfx}: {continent}/{entity} cq:{cq} itu:{itu}"
            )

    def check_callsign2(self, callsign):  # fixme
        if hasattr(self.look_up, "session"):
            if self.look_up.session:
                response = self.look_up.lookup(callsign)
                logger.debug(f"The Response: {response}\n")
                if response:
                    theirgrid = response.get("grid")
                    theircountry = response.get("country")
                    if self.pref.get("gridsquare"):
                        heading = bearing(self.pref.get("gridsquare"), theirgrid)
                        kilometers = distance(self.pref.get("gridsquare"), theirgrid)
                        self.heading_distance.setText(
                            f"heading {heading}° / distance {int(kilometers*0.621371)}mi {kilometers}km {theirgrid}"
                        )
                    # self.dx_entity.setText(f"{theircountry}")
                # else:
                # self.heading_distance.setText("Lookup failed.")
            ...

    def setmode(self, mode: str) -> None:
        if mode == "CW":
            self.mode.setText("CW")
            self.sent.setText("599")
            self.receive.setText("599")
            return
        if mode == "SSB":
            self.mode.setText("SSB")
            self.sent.setText("59")
            self.receive.setText("59")

    def get_opon(self):
        self.opon_dialog = OpOn()
        self.opon_dialog.accepted.connect(self.new_op)
        self.opon_dialog.open()

    def new_op(self):
        if self.opon_dialog.NewOperator.text():
            self.current_op = self.opon_dialog.NewOperator.text()

        self.opon_dialog.close()
        logger.debug(f"New Op: {self.current_op}")

    def poll_radio(self):
        if self.rig_control.online:
            vfo = self.rig_control.get_vfo()
            mode = self.rig_control.get_mode()
            logger.debug(f"VFO: {vfo}  MODE: {mode}")

    def read_cw_macros(self) -> None:
        """
        Reads in the CW macros, firsts it checks to see if the file exists. If it does not,
        and this has been packaged with pyinstaller it will copy the default file from the
        temp directory this is running from... In theory.
        """

        if not Path(DATA_PATH + "/cwmacros.txt").exists():
            logger.debug("read_cw_macros: copying default macro file.")
            copyfile(WORKING_PATH + "/data/cwmacros.txt", DATA_PATH + "/cwmacros.txt")
        with open(
            DATA_PATH + "/cwmacros.txt", "r", encoding="utf-8"
        ) as file_descriptor:
            for line in file_descriptor:
                try:
                    mode, fkey, buttonname, cwtext = line.split("|")
                    if mode.strip().upper() == "R" and self.pref.get("run_state"):
                        self.fkeys[fkey.strip()] = (buttonname.strip(), cwtext.strip())
                    if mode.strip().upper() != "R" and not self.pref.get("run_state"):
                        self.fkeys[fkey.strip()] = (buttonname.strip(), cwtext.strip())
                except ValueError as err:
                    ...
                    logger.info("read_cw_macros: %s", err)
        keys = self.fkeys.keys()
        if "F1" in keys:
            self.F1.setText(f"F1: {self.fkeys['F1'][0]}")
            self.F1.setToolTip(self.fkeys["F1"][1])
        if "F2" in keys:
            self.F2.setText(f"F2: {self.fkeys['F2'][0]}")
            self.F2.setToolTip(self.fkeys["F2"][1])
        if "F3" in keys:
            self.F3.setText(f"F3: {self.fkeys['F3'][0]}")
            self.F3.setToolTip(self.fkeys["F3"][1])
        if "F4" in keys:
            self.F4.setText(f"F4: {self.fkeys['F4'][0]}")
            self.F4.setToolTip(self.fkeys["F4"][1])
        if "F5" in keys:
            self.F5.setText(f"F5: {self.fkeys['F5'][0]}")
            self.F5.setToolTip(self.fkeys["F5"][1])
        if "F6" in keys:
            self.F6.setText(f"F6: {self.fkeys['F6'][0]}")
            self.F6.setToolTip(self.fkeys["F6"][1])
        if "F7" in keys:
            self.F7.setText(f"F7: {self.fkeys['F7'][0]}")
            self.F7.setToolTip(self.fkeys["F7"][1])
        if "F8" in keys:
            self.F8.setText(f"F8: {self.fkeys['F8'][0]}")
            self.F8.setToolTip(self.fkeys["F8"][1])
        if "F9" in keys:
            self.F9.setText(f"F9: {self.fkeys['F9'][0]}")
            self.F9.setToolTip(self.fkeys["F9"][1])
        if "F10" in keys:
            self.F10.setText(f"F10: {self.fkeys['F10'][0]}")
            self.F10.setToolTip(self.fkeys["F10"][1])
        if "F11" in keys:
            self.F11.setText(f"F11: {self.fkeys['F11'][0]}")
            self.F11.setToolTip(self.fkeys["F11"][1])
        if "F12" in keys:
            self.F12.setText(f"F12: {self.fkeys['F12'][0]}")
            self.F12.setToolTip(self.fkeys["F12"][1])


class EditMacro(QtWidgets.QDialog):
    """Change the current operator"""

    def __init__(self, function_key):
        self.function_key = function_key
        parent = None
        super().__init__(parent)
        uic.loadUi(WORKING_PATH + "/data/editmacro.ui", self)
        self.buttonBox.clicked.connect(self.store)
        self.macro_label.setText(function_key.text())
        self.the_macro.setText(function_key.toolTip())

    def store(self):
        """dialog magic"""
        ...
        # self.accept()


class OpOn(QtWidgets.QDialog):
    """Change the current operator"""

    def __init__(self, parent=None):
        super().__init__(parent)
        uic.loadUi(WORKING_PATH + "/data/opon.ui", self)
        self.buttonBox.clicked.connect(self.store)

    def store(self):
        """dialog magic"""
        ...
        # self.accept()


def load_fonts_from_dir(directory: str) -> set:
    """
    Well it loads fonts from a directory...
    """
    font_families = set()
    for _fi in QDir(directory).entryInfoList(["*.ttf", "*.woff", "*.woff2"]):
        _id = QFontDatabase.addApplicationFont(_fi.absoluteFilePath())
        font_families |= set(QFontDatabase.applicationFontFamilies(_id))
    return font_families


def install_icons():
    os.system(
        "xdg-icon-resource install --size 32 --context apps --mode user "
        f"{WORKING_PATH}/data/k6gte.not1mm-32.png k6gte-not1mm"
    )
    os.system(
        "xdg-icon-resource install --size 64 --context apps --mode user "
        f"{WORKING_PATH}/data/k6gte.not1mm-64.png k6gte-not1mm"
    )
    os.system(
        "xdg-icon-resource install --size 128 --context apps --mode user "
        f"{WORKING_PATH}/data/k6gte.not1mm-128.png k6gte-not1mm"
    )
    os.system(f"xdg-desktop-menu install {WORKING_PATH}/data/k6gte-not1mm.desktop")


def doimp(modname):
    return importlib.import_module(f"not1mm.plugins.{modname}")


def run():
    """
    Main Entry
    """

    install_icons()
    timer.start(1000)

    sys.exit(app.exec())


logger = logging.getLogger("__name__")
handler = logging.StreamHandler()
formatter = logging.Formatter(
    datefmt="%H:%M:%S",
    fmt="[%(asctime)s] %(levelname)s %(module)s - %(funcName)s Line %(lineno)d:\n%(message)s",
)
handler.setFormatter(formatter)
logger.addHandler(handler)

if Path("./debug").exists():
    # if True:
    logger.setLevel(logging.DEBUG)
    logger.debug("debugging on")
else:
    logger.setLevel(logging.WARNING)
    logger.warning("debugging off")

app = QtWidgets.QApplication(sys.argv)
app.setStyle("Fusion")
font_path = WORKING_PATH + "/data"
families = load_fonts_from_dir(os.fspath(font_path))
logger.info(families)
window = MainWindow()
window.setGeometry(-1, -1, 600, 200)
window.setWindowTitle(f"Not1MM v{__version__}")
window.show()
timer = QtCore.QTimer()
timer.timeout.connect(window.poll_radio)


if __name__ == "__main__":
    run()



from PyQt5 import QtWidgets

name = "ARRL Field Day"
mode = "BOTH"  # CW SSB BOTH RTTY

# 1 once per contest, 2 work each band, 3 each band/mode, 4 no dupe checking
dupe_type = 3


def interface(self):
    self.field1.hide()
    self.field2.hide()
    self.field3.show()
    self.field4.show()
    label = self.field3.findChild(QtWidgets.QLabel)
    label.setText("Class")
    label = self.field4.findChild(QtWidgets.QLabel)
    label.setText("Section")



from PyQt5 import QtWidgets

name = "Winter Field Day"


def interface(self):
    self.field1.hide()
    self.field2.hide()
    self.field3.show()
    self.field4.show()
    label = self.field3.findChild(QtWidgets.QLabel)
    label.setText("Class")
    label = self.field4.findChild(QtWidgets.QLabel)
    label.setText("Section")



"""
callsign lookup classes for:
QRZ
HamDB
HamQTH
"""

import logging
import xmltodict
import requests


class HamDBlookup:
    """
    Class manages HamDB lookups.
    """

    def __init__(self) -> None:
        self.logger = logging.getLogger("__name__")
        self.url = "https://api.hamdb.org/"
        self.error = False

    def lookup(self, call: str) -> tuple:
        """
        Lookup a call on QRZ

        <?xml version="1.0" encoding="utf-8"?>
        <hamdb version="1.0">
        <callsign>
        <call>K6GTE</call>
        <class>G</class>
        <expires>11/07/2027</expires>
        <grid>DM13at</grid>
        <lat>33.8254731</lat>
        <lon>-117.9875229</lon>
        <status>A</status>
        <fname>Michael</fname>
        <mi>C</mi>
        <name>Bridak</name>
        <suffix/>
        <addr1>2854 W Bridgeport Ave</addr1>
        <addr2>Anaheim</addr2>
        <state>CA</state>
        <zip>92804</zip>
        <country>United States</country>
        </callsign>
        <messages>
        <status>OK</status>
        </messages>
        </hamdb>
        """

        self.logger.info("%s", call)
        grid = False
        name = False
        error_text = False
        nickname = False

        try:
            self.error = False
            query_result = requests.get(
                self.url + call + "/xml/wfd_logger", timeout=10.0
            )
        except requests.exceptions.Timeout as exception:
            self.error = True
            return grid, name, nickname, exception
        if query_result.status_code == 200:
            self.error = False
            rootdict = xmltodict.parse(query_result.text)
            root = rootdict.get("hamdb")
            if root:
                messages = root.get("messages")
                callsign = root.get("callsign")
            if messages:
                error_text = messages.get("status")
                self.logger.debug("HamDB: %s", error_text)
                if error_text != "OK":
                    self.error = False
            if callsign:
                self.logger.debug("HamDB: found callsign field")
                if callsign.get("grid"):
                    grid = callsign.get("grid")
                if callsign.get("fname"):
                    name = callsign.get("fname")
                if callsign.get("name"):
                    if not name:
                        name = callsign.get("name")
                    else:
                        name = f"{name} {callsign.get('name')}"
                if callsign.get("nickname"):
                    nickname = callsign.get("nickname")
        else:
            self.error = True
            error_text = str(query_result.status_code)
        self.logger.info("HamDB-lookup: %s %s %s %s", grid, name, nickname, error_text)
        return grid, name, nickname, error_text


class QRZlookup:
    """
    Class manages QRZ lookups. Pass in a username and password at instantiation.
    """

    def __init__(self, username: str, password: str) -> None:
        self.logger = logging.getLogger("__name__")
        self.session = False
        self.expiration = False
        self.error = (
            False  # "password incorrect", "session timeout", and "callsign not found".
        )
        self.username = username
        self.password = password
        self.qrzurl = "https://xmldata.qrz.com/xml/134/"
        self.message = False
        self.lastresult = False
        self.getsession()

    def getsession(self) -> None:
        """
        Get QRZ session key.
        Stores key in class variable 'session'
        Error messages returned by QRZ will be in class variable 'error'
        Other messages returned will be in class variable 'message'

        <?xml version="1.0" ?>
        <QRZDatabase version="1.34">
        <Session>
            <Key>2331uf894c4bd29f3923f3bacf02c532d7bd9</Key>
            <Count>123</Count>
            <SubExp>Wed Jan 1 12:34:03 2013</SubExp>
            <GMTime>Sun Aug 16 03:51:47 2012</GMTime>
        </Session>
        </QRZDatabase>

        Session section fields
        Field	Description
        Key	a valid user session key
        Count	Number of lookups performed by this user in the current 24 hour period
        SubExp	time and date that the users subscription will expire - or - "non-subscriber"
        GMTime	Time stamp for this message
        Message	An informational message for the user
        Error	XML system error message
        """
        self.logger.info("QRZlookup-getsession:")
        self.error = False
        self.message = False
        self.session = False
        try:
            payload = {"username": self.username, "password": self.password}
            query_result = requests.get(self.qrzurl, params=payload, timeout=10.0)
            baseroot = xmltodict.parse(query_result.text)
            root = baseroot.get("QRZDatabase")
            if root:
                session = root.get("Session")
            self.logger.info("\n\n%s\n\n", root)
            if session.get("Key"):
                self.session = session.get("Key")
            if session.get("SubExp"):
                self.expiration = session.get("SubExp")
            if session.get("Error"):
                self.error = session.get("Error")
            if session.get("Message"):
                self.message = session.get("Message")
            self.logger.info(
                "key:%s error:%s message:%s",
                self.session,
                self.error,
                self.message,
            )
        except requests.exceptions.RequestException as exception:
            self.logger.info("%s", exception)
            self.session = False
            self.error = f"{exception}"

    def lookup(self, call: str) -> tuple:
        """
        Lookup a call on QRZ
        """
        self.logger.info("%s", call)
        response = None
        if self.session:
            payload = {"s": self.session, "callsign": call}
            try:
                query_result = requests.get(self.qrzurl, params=payload, timeout=10.0)
            except requests.exceptions.Timeout as exception:
                self.error = True
                return {"error": exception}
            baseroot = xmltodict.parse(query_result.text)
            self.logger.debug(f"xml lookup {baseroot}\n")
            root = baseroot.get("QRZDatabase")
            self.logger.info("\n\n%s\n\n", root)
            session = root.get("Session")
            if not session.get("Key"):  # key expired get a new one
                self.logger.info("no key, getting new one.")
                self.getsession()
                if self.session:
                    payload = {"s": self.session, "callsign": call}
                    query_result = requests.get(
                        self.qrzurl, params=payload, timeout=3.0
                    )
                    baseroot = xmltodict.parse(query_result.text)
                    root = baseroot.get("QRZDatabase")
            # grid, name, nickname, error_text = self.parse_lookup(query_result)
        # self.logger.info("%s %s %s %s", grid, name, nickname, error_text)
        return root.get("Callsign")

    def parse_lookup(self, query_result):
        """
        Returns gridsquare and name for a callsign looked up by qrz or hamdb.
        Or False for both if none found or error.

        <?xml version="1.0" encoding="utf-8"?>
        <QRZDatabase version="1.34" xmlns="http://xmldata.qrz.com">
        <Callsign>
        <call>K6GTE</call>
        <aliases>KM6HQI</aliases>
        <dxcc>291</dxcc>
        <nickname>Mike</nickname>
        <fname>Michael C</fname>
        <name>Bridak</name>
        <addr1>2854 W Bridgeport Ave</addr1>
        <addr2>Anaheim</addr2>
        <state>CA</state>
        <zip>92804</zip>
        <country>United States</country>
        <lat>33.825460</lat>
        <lon>-117.987510</lon>
        <grid>DM13at</grid>
        <county>Orange</county>
        <ccode>271</ccode>
        <fips>06059</fips>
        <land>United States</land>
        <efdate>2021-01-13</efdate>
        <expdate>2027-11-07</expdate>
        <class>G</class>
        <codes>HVIE</codes>
        <email>michael.bridak@gmail.com</email>
        <u_views>1569</u_views>
        <bio>6399</bio>
        <biodate>2022-02-26 00:51:44</biodate>
        <image>https://cdn-xml.qrz.com/e/k6gte/qsl.png</image>
        <imageinfo>285:545:99376</imageinfo>
        <moddate>2021-04-08 21:41:07</moddate>
        <MSA>5945</MSA>
        <AreaCode>714</AreaCode>
        <TimeZone>Pacific</TimeZone>
        <GMTOffset>-8</GMTOffset>
        <DST>Y</DST>
        <eqsl>0</eqsl>
        <mqsl>1</mqsl>
        <cqzone>3</cqzone>
        <ituzone>6</ituzone>
        <born>1967</born>
        <lotw>1</lotw>
        <user>K6GTE</user>
        <geoloc>geocode</geoloc>
        <name_fmt>Michael C "Mike" Bridak</name_fmt>
        </Callsign>
        <Session>
        <Key>42d5c9736525b485e8edb782b101c74b</Key>
        <Count>4140</Count>
        <SubExp>Tue Feb 21 07:01:49 2023</SubExp>
        <GMTime>Sun May  1 20:00:36 2022</GMTime>
        <Remark>cpu: 0.022s</Remark>
        </Session>
        </QRZDatabase>

        """
        self.logger.info("QRZlookup-parse_lookup:")
        grid = False
        name = False
        error_text = False
        nickname = False
        if query_result.status_code == 200:
            baseroot = xmltodict.parse(query_result.text)
            root = baseroot.get("QRZDatabase")
            session = root.get("Session")
            callsign = root.get("Callsign")
            self.logger.info("\n\n%s\n\n", root)
            if session.get("Error"):
                error_text = session.get("Error")
                self.error = error_text
            if callsign:
                if callsign.get("grid"):
                    grid = callsign.get("grid")
                if callsign.get("fname"):
                    name = callsign.get("fname")
                if callsign.get("name"):
                    if not name:
                        name = callsign.get("name")
                    else:
                        name = f"{name} {callsign.get('name')}"
                if callsign.get("nickname"):
                    nickname = callsign.get("nickname")
        self.logger.info("%s %s %s %s", grid, name, nickname, error_text)
        return grid, name, nickname, error_text


class HamQTH:
    """HamQTH lookup"""

    def __init__(self, username: str, password: str) -> None:
        """initialize HamQTH lookup"""
        self.logger = logging.getLogger("__name__")
        self.username = username
        self.password = password
        self.url = "https://www.hamqth.com/xml.php"
        self.session = False
        self.error = False
        self.getsession()

    def getsession(self) -> None:
        """get a session key"""
        self.logger.info("Getting session")
        self.error = False
        self.session = False
        payload = {"u": self.username, "p": self.password}
        try:
            query_result = requests.get(self.url, params=payload, timeout=10.0)
        except requests.exceptions.Timeout:
            self.error = True
            return
        self.logger.info("resultcode: %s", query_result.status_code)
        baseroot = xmltodict.parse(query_result.text)
        root = baseroot.get("HamQTH")
        session = root.get("session")
        if session:
            if session.get("session_id"):
                self.session = session.get("session_id")
            if session.get("error"):
                self.error = session.get("error")
        self.logger.info("session: %s", self.session)

    def lookup(self, call: str) -> tuple:
        """
        Lookup a call on HamQTH
        """
        grid, name, nickname, error_text = False, False, False, False
        if self.session:
            payload = {"id": self.session, "callsign": call, "prg": "wfdlogger"}
            try:
                query_result = requests.get(self.url, params=payload, timeout=10.0)
            except requests.exceptions.Timeout as exception:
                self.error = True
                return grid, name, nickname, exception
            self.logger.info("resultcode: %s", query_result.status_code)
            baseroot = xmltodict.parse(query_result.text)
            root = baseroot.get("HamQTH")
            search = root.get("search")
            session = root.get("session")
            if not search:
                if session:
                    if session.get("error"):
                        if session.get("error") == "Callsign not found":
                            error_text = session.get("error")
                            return grid, name, nickname, error_text
                        if session.get("error") == "Session does not exist or expired":
                            self.getsession()
                            query_result = requests.get(
                                self.url, params=payload, timeout=10.0
                            )
            grid, name, nickname, error_text = self.parse_lookup(root)
        self.logger.info("%s %s %s %s", grid, name, nickname, error_text)
        return grid, name, nickname, error_text

    def parse_lookup(self, root) -> tuple:
        """
        Returns gridsquare and name for a callsign looked up by qrz or hamdb.
        Or False for both if none found or error.
        """
        grid, name, nickname, error_text = False, False, False, False
        session = root.get("session")
        search = root.get("search")
        if session:
            if session.get("error"):
                error_text = session.get("error")
        if search:
            if search.get("grid"):
                grid = search.get("grid")
            if search.get("nick"):
                nickname = search.get("nick")
            if search.get("adr_name"):
                name = search.get("adr_name")
        return grid, name, nickname, error_text


def main():
    """Just in case..."""
    print("I'm not a program.")


if __name__ == "__main__":
    main()



#!/usr/bin/env python3
"""
K6GTE Contest logger
Email: michael.bridak@gmail.com
GPL V3
"""

# pylint: disable=too-many-lines
# pylint: disable=invalid-name
# pylint: disable=no-member
# pylint: disable=no-name-in-module
# pylint: disable=c-extension-no-member
# pylint: disable=unused-import

from math import radians, sin, cos, atan2, sqrt, asin, pi
import sys
import socket
import os
import logging
import threading
import uuid
import queue
import time
from itertools import chain

from json import dumps, loads, JSONDecodeError
from datetime import datetime, timedelta
from pathlib import Path
from shutil import copyfile
import toml

import requests

try:
    from not1mm.lib.database import DataBase
    from not1mm.lib.version import __version__
except ModuleNotFoundError:
    from lib.database import DataBase
    from lib.version import __version__


class main:
    """test"""

    def __init__(self):
        # create the DB
        # self.db = DataBase("file::memory:?cache=shared")
        self.db = DataBase("testdb.db")
        self.db_object = None

    def run(self):
        """run"""
        # Set persistent values for contact
        self.db.empty_contact["app"] = "K6GTELogger"
        self.db.empty_contact["mycall"] = "K6GTE"
        self.db.empty_contact["operator"] = "K6GTE"
        self.db.empty_contact["contestname"] = "CQWWCW"
        self.db.empty_contact["contestnr"] = "1"
        self.db.empty_contact["StationName"] = "20M CW"

        # get clean contact object with persistent changes from above
        self.db_object = self.db.get_empty()

        # Make changes to save to the DB
        self.db_object["ID"] = uuid.uuid4().hex
        self.db_object["timestamp"] = datetime.utcnow().strftime("%Y-%m-%d %H:%M:%S")
        self.db_object["band"] = "20"
        self.db_object["rxfreq"] = ""
        self.db_object["txfreq"] = ""
        self.db_object["countryprefix"] = "K"
        self.db_object["continent"] = "NA"
        self.db_object["snt"] = "599"
        self.db_object["sntnr"] = "1"
        self.db_object["rcv"] = "599"
        self.db_object["rcvnr"] = "37"
        self.db_object["gridsquare"] = "DM13at"
        self.db_object["section"] = "ORG"
        self.db_object["qth"] = "Their Home"
        self.db_object["name"] = "Russ"
        self.db_object["power"] = "100"
        self.db_object["points"] = "2"
        self.db_object["call"] = "K5TUX"
        self.db_object["mode"] = "CW"

        # Save the contact to the DB
        self.db.log_contact(self.db_object)
        print(f"\nSaved Data:\n{self.db.fetch_all_contacts_asc()}\n\n")

        # save changes
        self.db_object["rxfreq"] = "1402700"
        self.db.change_contact(self.db_object)
        print(f"\nUpdated Data:\n{self.db.fetch_all_contacts_asc()}\n\n")

        # get unique id of record
        uniqid = self.db.get_unique_id(1)
        print(f"Unique ID: {uniqid}")

        # fetch descending
        print(f"\nSorted Descending Data:\n{self.db.fetch_all_contacts_desc()}\n\n")

        # fetch last record
        print(f"\nGet last contact:\n{self.db.fetch_last_contact()}\n\n")

        # fetch all dirty
        print(f"\nGet all marked dirty:\n{self.db.fetch_all_dirty_contacts()}\n\n")

        # fetch all dirty
        print(f"\nDeleting Contact:\n{self.db.delete_contact(1)}\n\n")

    def tomltest(self):
        with open("070TPRTTY.udc", "r") as fd:
            toml_string = toml.loads(fd.read())
        print(toml_string)


if __name__ == "__main__":
    app = main()
    app.run()
    app.tomltest()
